<?php
/**
 * Plugin Name: WELPP v1.0.4
 * Plugin URI: https://welpp.nl/welppdesk/#Handleiding-WooCommerce-plugin
 * Description: Connects WooCommerce to WELPP REST API V1.0.4
 * Version: 1.0.4
 * Author: WELPP
 * Author URI: https://welpp.nl
 * Requires at least: 3.5
 *
 * Text Domain: WELPP
 *
 * @package WelppWebservice
 * @category Core
 * @author WELPP
 */

namespace Wics;

// Define the base path
define(__NAMESPACE__ . '\BASEPATH', dirname(__FILE__).'/');
define(__NAMESPACE__ . '\VERSION', 1);


if ( !defined('ABSPATH') ) {
	require_once(realpath('../../../wp-load.php'));
}



class WelppModule
{
	protected static $instance;
	public $logIsEnabled = false;
	public static function getInstance()
	{
		if (!isset(self::$instance)) {
			self::$instance = new WelppModule;
				
		}
		return self::$instance;
	}

	// Add actions
	public function __construct()
	{
		
		// Autoloader
		spl_autoload_register(array($this, '_autoload'));
		
		add_action('admin_menu', array($this, 'register_admin_page'));
		add_action('woocommerce_order_status_processing', array($this, 'saveOrder'), 99999999, 1);
		
		// todo: check moment
		add_action('wp', array($this, 'cronstarter_activation'));
	
		
		register_deactivation_hook (__FILE__, array($this, 'cronstarter_deactivate'));
		// update stock twice each day
		add_action('update_stock_cronjob', array($this, 'updateStock'));
		
		// update order with shipment detail each 15 minutes.
		add_action('update_order_cronjob', array($this, 'updateOrder'));
		
		// custom minutes
		add_filter( 'cron_schedules', array($this, 'cron_add_minute' ));
			
		// add column for export to Welpp on order page
		add_action('manage_edit-shop_order_columns', array($this, 'add_order_status_column_header'));
		add_action('manage_shop_order_posts_custom_column', array($this, 'add_order_status_column_content'));
		add_action('manage_shop_order_posts_custom_column', array($this, 'wpse_load_plugin_css'));
		
		// retry button
		add_action( 'woocommerce_order_action_wc_wics_wms_connector_export_order', array( $this, 'process_order_meta_box_actions' ) );
		add_action( 'wp_ajax_wc_wics_wms_connector_export_order', array( $this, 'process_ajax_order_action' ) );
		add_action('woocommerce_admin_order_actions_end', array($this, 'retry_sent_to_welpp'));
	
		// add shipment details on order page
		add_action('add_meta_boxes', array($this, 'mv_add_boxes'));
		
		$this->setLogEnabled();
		
	}
	
	function setLogEnabled(){
		$this->logIsEnabled = true;
	}
	
	
	function mv_add_boxes(){
		global $post;
		
		$provider = get_post_meta($post->ID, '_order_track_provider', true);
		$trackAndTrace = get_post_meta($post->ID, '_order_track', true);
		$trackAndTraceUrl = get_post_meta($post->ID, '_order_track_url', true);
		
		$postShipmentDetails = array("provider"=> $provider, "trackAndTrace" =>  $trackAndTrace, "trackAndTraceUrl" => $trackAndTraceUrl);
		
		if($this->hasShipmentDetails($postShipmentDetails)){
			add_meta_box( 'mv_shipment_fields', __('Shipment Details','welpp'), array(&$this, 'mv_add_shipment_fields'), 'shop_order', 'side', 'core', $postShipmentDetails );
		}
	}
	function hasShipmentDetails($postShipmentDetails){
		return $postShipmentDetails["provider"] != "" && $postShipmentDetails["trackAndTrace"] != "" && $postShipmentDetails["trackAndTraceUrl"] != "";
	}
	
	function mv_add_shipment_fields($post, $arguments){
		
		$html  = "<label><b>Provider:</b></label> <br/>";
		$html .= "<label>" . $arguments["args"]["provider"] . "</label><br/><br/>";
		$html .= "<label><b>Track and trace code:</b></label><br/>";
		$html .= "<label>" .$arguments["args"]["trackAndTrace"] . "</label><br/><br/>";
		$html .= "<label><b>Track and trace url:</b></label><br/>";
		$html .= "<a href=".  $arguments["args"]["trackAndTraceUrl"]  .   " target='_blank' title=". $trackAndTraceUrl. ">" . substr($arguments["args"]["trackAndTraceUrl"], 0, 35). "...</a>";		
		
		printf($html);
		
	}

	
	function wpse_load_plugin_css() {
		$plugin_url = plugin_dir_url( __FILE__ );
		wp_enqueue_style( 'welpp_css', $plugin_url . 'css/style.css' );
	}
	
	function add_order_status_column_header($columns){
	
		$new_columns = array();
	
		foreach ( $columns as $column_name => $column_info ) {
			$new_columns[ $column_name ] = $column_info;
	
			if ( 'order_status' == $column_name ) {
				$new_columns['wics_wms_export_status'] = __( 'Export Status', "Welpp" );
			}
		}
	
		return $new_columns;
	}
	
	function add_order_status_column_content($column){
		global $post;
	
		if($column == 'wics_wms_export_status'){
				
			$order = new \WC_Order( $post->ID );
			$sent_to_welpp = get_post_meta($post->ID, '_sent_to_welpp', true);
			$response_welpp = get_post_meta($post->ID, '_response_welpp', true);
			$icon_style = "";
			$data_tip = "";
			if(isset($sent_to_welpp)){
				if($sent_to_welpp == ""){
					printf('not sent');
				} else {
					if($sent_to_welpp == true){
						$icon_style = "mark wics_wms_exported";
						$data_tip = "Exported: " . $response_welpp;
							
					} else if($sent_to_welpp == false){
						$icon_style = "mark wics_wms_not_exported";
						$data_tip = "Not exported: " . $response_welpp;
					}
					printf( '<mark class="%1$s tips" data-tip="%2$s">%2$s</mark>', $icon_style, $data_tip );
				}
			}
	
		}
	}
	
	function  process_order_meta_box_actions($order){
		$this->saveOrder($order->id);
	}
	
	function retry_sent_to_welpp($column){
		
		$sent_to_welpp = get_post_meta($column->id, '_sent_to_welpp', true);
		
		if(isset($sent_to_welpp)){
			if($sent_to_welpp != "" && $sent_to_welpp == false && $column->post_status == "wc-processing"){
				printf("<a class='button tips' name='retry' style='font-size:xx-small;' href='%s' >Export</a>", admin_url() . "admin-ajax.php?action=wc_wics_wms_connector_export_order&id=" . $column->id );
			}
		}
	}
	
	function process_ajax_order_action(){
		
		if(isset($_GET['id'])){
			$this->saveOrder($_GET['id']);
		}
		
		wp_redirect( wp_get_referer() );
	}
	
	function cronstarter_activation() {
		if( !wp_next_scheduled( 'update_stock_cronjob' ) ) {
			wp_schedule_event( time(), 'twicedaily', 'update_stock_cronjob' );
			$this->log("Cron job updatecronjob has been created.");
		} 
		if(!wp_next_scheduled('update_order_cronjob')){
			//custom minutes
			wp_schedule_event( time(), 'everyfiveteenminutes', 'update_order_cronjob' );
			$this->log("Cron job update_order_cronjob has been created.");
		}
		
	}
	
	// add custom interval
	function cron_add_minute( $schedules ) {
		// Adds once every minute to the existing schedules.
		$schedules['everyfiveteenminutes'] = array(
				'interval' => 630,
				'display' => __( 'Once Every 15 Minutes' )
		);
		return $schedules;
	}
	
	function cronstarter_deactivate() {
		// find out when the last event was scheduled
		$timestamp = wp_next_scheduled ('update_stock_cronjob');
		// unschedule previous event if any
		wp_unschedule_event ($timestamp, 'update_stock_cronjob');
		$this->log("Cron job update_stock_cronjob has been removed. " . $timestamp);
		
		// find out when the last event was scheduled
		$timestamp = wp_next_scheduled ('update_order_cronjob');
		// unschedule previous event if any
		wp_unschedule_event ($timestamp, 'update_order_cronjob');
		$this->log("Cron job update_order_cronjob has been removed. " . $timestamp);

		
	}
	
	public function saveOrder($order_id)
	{
		if(!$this->orderExistsInWelpp($order_id))
		{
			$order = new \WC_Order($order_id);
			$wicsOrder = new \Wics\Model\Order($order);
				
			$sent = \Wics\Webservice::getInstance()->saveOrder($wicsOrder);
			if($sent->success == true){
				$this->log(sprintf("Order with id %d was sent to welpp. Response: " . $sent->message, $order_id));
				update_post_meta($order_id, "_sent_to_welpp", 1);
				update_post_meta($order_id, "_response_welpp", $sent->message);
			} else if($sent->success == false){
				$this->log(sprintf("Order with id %d failed to sent to welpp. Response: " . $sent->message, $order_id));
				update_post_meta($order_id, "_sent_to_welpp", 0);
				update_post_meta($order_id, "_response_welpp", $sent->message);
			}
		} else {
			$this->log(sprintf("Order with id %d was not sent to welpp, it already exists." , $order_id));
		}
			
	}
	
	
	
	function updateOrder(){
		$updateOrder = new UpdateOrder();
		$updateOrder->update();
	}
	
	function updateStock(){
		$updateStock = new UpdateStock();
		$updateStock->update();
	}
	
	public function orderExistsInWelpp($order_id){
		$sent_to_welpp = get_post_meta($order_id, '_sent_to_welpp', true);
		return $sent_to_welpp != "" && $sent_to_welpp == true;
	}
	
	public function log( $message ) {
		if($this->logIsEnabled == true){
			error_log($message);
		}
		
	}
	
	private function _autoload($class)
	{
		$classFile = BASEPATH.'classes'.str_replace(array('Wics\\','\\'), '/', $class).'.php';
		if (preg_match('/^Wics/Usi', $class) && file_exists($classFile)) {
			require_once($classFile);
		}
	}
	
	
	public function register_admin_page()
	{
		add_menu_page('Welpp Integration plugin', 'WELPP', 'manage_options', \Wics\BASEPATH.'templates/admin-page.php', '', '', 101);
	}
	
	
	function viewCronJobs(){
		echo '<pre>'; print_r( _get_cron_array() ); echo '</pre>';
		echo "count: " . count(_get_cron_array());
		
	}
	
	
	

}

$wics = WelppModule::getInstance();